(* File generated from pplite.idl *)


(*
 This file is part of the APRON Library, released under GPL license.
 Please read the COPYING file packaged in the distribution.
*)

(** Polyhedra abstract domains (PPLite wrapper) *)

type loose
type strict
  (** Two flavors for convex polyhedra: loose or strict.

      Loose polyhedra cannot have strict inequality constraints like [x>0].
      Convex polyhedra are defined by the conjunction of a set of linear
      constraints of the form
      [a_0*x_0 + ... + a_n*x_n + b >= 0] or
      [a_0*x_0 + ... + a_n*x_n + b > 0]
      where [a_0, ..., a_n, b, c] are constants and [x_0, ..., x_n] variables.
  *)

type 'a t
(** Type of convex polyhedra, where ['a] is [loose] or [strict].

    Abstract values which are convex polyhedra have the type
    [loose t Apron.AbstractX.t] or [strict t Apron.AbstractX.t].
    Managers allocated by PPLite have the type ['a t Apron.Manager.t].
*)

external manager_alloc_loose : unit -> loose t Apron.Manager.t
	= "camlidl_pplite_ap_pplite_manager_alloc_loose"

external manager_alloc_strict : unit -> strict t Apron.Manager.t
	= "camlidl_pplite_ap_pplite_manager_alloc_strict"

external manager_set_kind : 'a t Apron.Manager.t -> string -> unit
	= "camlidl_pplite_ap_pplite_manager_set_kind"

external manager_get_kind : 'a t Apron.Manager.t -> string
	= "camlidl_pplite_ap_pplite_manager_get_kind"

external manager_set_widen_spec : 'a t Apron.Manager.t -> string -> unit
	= "camlidl_pplite_ap_pplite_manager_set_widen_spec"

external manager_get_widen_spec : 'a t Apron.Manager.t -> string
	= "camlidl_pplite_ap_pplite_manager_get_widen_spec"

external abstract0_split : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> Apron.Lincons0.t -> bool -> bool -> 'a t Apron.Abstract0.t
	= "camlidl_pplite_ap_pplite_abstract0_split"

external abstract0_is_disjunctive : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> bool
	= "camlidl_pplite_ap_pplite_abstract0_is_disjunctive"

external abstract0_num_disjuncts : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> int
	= "camlidl_pplite_ap_pplite_abstract0_num_disjuncts"

external abstract0_disj_to_lincons_array : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> int -> Apron.Lincons0.t array
	= "camlidl_pplite_ap_pplite_abstract0_disj_to_lincons_array"

external abstract0_disj_to_tcons_array : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> int -> Tcons0.t array
	= "camlidl_pplite_ap_pplite_abstract0_disj_to_tcons_array"

external abstract0_geom_subseteq : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> 'a t Apron.Abstract0.t -> bool
	= "camlidl_pplite_ap_pplite_abstract0_geom_subseteq"

external abstract0_collapse : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> int -> unit
	= "camlidl_pplite_ap_pplite_abstract0_collapse"


let manager_is_pplite man =
  let str = Apron.Manager.get_library man in
  let str =
    try String.sub str 0 6
    with Invalid_argument _ -> ""
  in
  (String.compare str "PPLite")==0
let manager_of_pplite (man:'a t Apron.Manager.t) : 'b Apron.Manager.t = Obj.magic man
let manager_to_pplite (man:'a Apron.Manager.t) : 'b t Apron.Manager.t =
  if manager_is_pplite man then
    Obj.magic man
  else
    failwith "Pplite.to_pplite: the argument manager is not a PPLite manager"

let manager_is_pplite_loose man =
  let str = Apron.Manager.get_library man in
  (String.compare str "PPLite polyhedra, loose mode")==0
let manager_of_pplite_loose (man:loose t Apron.Manager.t) : 'a Apron.Manager.t = Obj.magic man
let manager_to_pplite_loose (man:'a Apron.Manager.t) : loose t Apron.Manager.t =
  if manager_is_pplite_loose man then
    Obj.magic man
  else
    failwith "Pplite.to_pplite_loose: the argument manager is not a loose PPLite manager"

let manager_is_pplite_strict man =
  let str = Apron.Manager.get_library man in
  (String.compare str "PPLite polyhedra, strict mode")==0
let manager_of_pplite_strict (man:strict t Apron.Manager.t) : 'a Apron.Manager.t = Obj.magic man
let manager_to_pplite_strict (man:'a Apron.Manager.t) : strict t Apron.Manager.t =
  if manager_is_pplite_strict man then
    Obj.magic man
  else
    failwith "Pplite.to_pplite_strict: the argument manager is not a strict PPLite manager"

module Abstract0 = struct
  let is_pplite abs =
    manager_is_pplite (Apron.Abstract0.manager abs)
  let is_pplite_loose abs =
    manager_is_pplite_loose (Apron.Abstract0.manager abs)
  let is_pplite_strict abs =
    manager_is_pplite (Apron.Abstract0.manager abs)
  let of_pplite (abs: 'a t Apron.Abstract0.t) : 'b Apron.Abstract0.t = Obj.magic abs
  let of_pplite_loose (abs: loose t Apron.Abstract0.t) : 'a Apron.Abstract0.t = Obj.magic abs
  let of_pplite_strict (abs: strict t Apron.Abstract0.t) : 'a Apron.Abstract0.t = Obj.magic abs
  let to_pplite (abs:'a Apron.Abstract0.t) : 'b t Apron.Abstract0.t =
    if is_pplite abs then
      Obj.magic abs
    else
      failwith "Pplite.Abstract0.to_pplite: the argument value is not a PPLite value"
  let to_pplite_loose (abs:'a Apron.Abstract0.t) : loose t Apron.Abstract0.t =
    if is_pplite_loose abs then
      Obj.magic abs
    else
      failwith "Pplite.Abstract0.to_pplite_loose: the argument value is not a loose pplite value"
  let to_pplite_strict (abs:'a Apron.Abstract0.t) : strict t Apron.Abstract0.t =
    if is_pplite_strict abs then
      Obj.magic abs
    else
      failwith "Pplite.Abstract0.to_pplite_strict: the argument value is not a strict pplite value"
end

module Abstract1 = struct
  let is_pplite abs =
    manager_is_pplite (Apron.Abstract1.manager abs)
  let is_pplite_loose abs =
    manager_is_pplite_loose (Apron.Abstract1.manager abs)
  let is_pplite_strict abs =
    manager_is_pplite (Apron.Abstract1.manager abs)
  let of_pplite (abs: 'a t Apron.Abstract1.t) : 'b Apron.Abstract1.t = Obj.magic abs
  let of_pplite_loose (abs: loose t Apron.Abstract1.t) : 'a Apron.Abstract1.t = Obj.magic abs
  let of_pplite_strict (abs: strict t Apron.Abstract1.t) : 'a Apron.Abstract1.t = Obj.magic abs
  let to_pplite (abs:'a Apron.Abstract1.t) : 'b t Apron.Abstract1.t =
    if is_pplite abs then
      Obj.magic abs
    else
      failwith "Pplite.Abstract1.to_pplite: the argument value is not a pplite value"
  let to_pplite_loose (abs:'a Apron.Abstract1.t) : loose t Apron.Abstract1.t =
    if is_pplite_loose abs then
      Obj.magic abs
    else
      failwith "Pplite.Abstract1.to_pplite_loose: the argument value is not a loose pplite value"
  let to_pplite_strict (abs:'a Apron.Abstract1.t) : strict t Apron.Abstract1.t =
    if is_pplite_strict abs then
      Obj.magic abs
    else
      failwith "Pplite.Abstract1.to_pplite_strict: the argument value is not a strict pplite value"
end

